import {Command} from 'ckeditor5/src/core';
import {  getClosestSelectedSpanElement, getAttrsData, getRangesOfBlockGroups, checkCanBeQuoted, findQuote } from "./utils"; 
import { first } from 'ckeditor5/src/utils';


import { DocumentFragment, Element, Position, Range, Schema, Writer } from 'ckeditor5/src/engine';

const SPAN = 'span';

export default class SpanCommand extends Command {


  execute(settings) { 
    const model = this.editor.model;
    const schema = model.schema;
    const selection = model.document.selection;
    const existingSpan = getClosestSelectedSpanElement(selection);
    
    if(!settings)
      settings = {};

    var content = settings.content? settings.content :'';
    var id = settings.id? settings.id :'';
    var className = settings.className? settings.className :'';
    var langcode = settings.langcode? settings.langcode :'';
    var rawStyles = settings.rawStyles? settings.rawStyles :'';
    var title = settings.title? settings.title :'';

    var attributes = { class: className, id: id, lang: langcode, style: rawStyles, title: title };


		const blocks = Array.from( selection.getSelectedBlocks() );

      model.change((writer) => {

        if(existingSpan)
        {
          writer.setAttributes(attributes, existingSpan);
        }
        else 
        {
          const docFrag = this.editor.model.change( writer => {
            const spanElt = writer.createElement( SPAN, attributes);
            const docFrag = writer.createDocumentFragment();
  
            writer.append( spanElt, docFrag );
            writer.insertText( content, spanElt );
  
            return docFrag;
        } );
  
        this.editor.model.insertContent( docFrag );
  
        }
      });
  }


  refresh() {
    this.value = this._getValue();  
    //this.isEnabled = true;
    console.log('refresh', this.value );
    this.isEnabled = this._checkEnabled();
  }

  /**
   * Checks whether the command can be enabled in the current context.
   *
   * @returns Whether the command should be enabled.
   */
  _checkEnabled() {
    const selection = this.editor.model.document.selection;
    const existingSpan = getClosestSelectedSpanElement(selection);

    if ( this.value ) {
      return true;
    }
    if(existingSpan) {
      const startOffset = existingSpan.startOffset;
      const data = existingSpan.getChild(0).data;  
      return data;
    }
    return false;
  }

  /**
   * Checks the command's {@link #value}.
   */
  _getValue() {
    const selection = this.editor.model.document.selection;
    const existingSpan = getClosestSelectedSpanElement(selection);
    if(existingSpan) {
      const startOffset = existingSpan.startOffset;
      const data = existingSpan.getChild(0).data;  
      return data;
    }
    return false;

    //const firstBlock = first( selection.getSelectedBlocks() );
    //return !!( firstBlock && findQuote( firstBlock ) );
  }

}

